@@ -2,11 +2,11 @@ package RedisDB;
 
 use strict;
 use warnings;
-our $VERSION = "2.13_06";
+our $VERSION = "2.19_01";
 $VERSION = eval $VERSION;
 
 use RedisDB::Error;
-use RedisDB::Parse::Redis;
+use RedisDB::Parser;
 use IO::Socket::INET;
 use IO::Socket::UNIX;
 use Socket qw(MSG_DONTWAIT MSG_NOSIGNAL SO_RCVTIMEO SO_SNDTIMEO);
@@ -160,7 +160,11 @@ sub new {
 
 sub _init_parser {
     my $self = shift;
-    $self->{_parser} = RedisDB::Parse::Redis->new( utf8 => $self->{utf8}, redisdb => $self );
+    $self->{_parser} = RedisDB::Parser->new(
+        utf8        => $self->{utf8},
+        master      => $self,
+        error_class => 'RedisDB::Error',
+    );
 }
 
 =head2 $self->execute($command, @arguments)
@@ -284,7 +288,16 @@ sub _connect {
 
     if ( $self->{timeout} ) {
         my $timeout;
-        if ( $Config{osname} eq 'netbsd' and $Config{osvers} >= 6.0 and $Config{longsize} == 4 ) {
+
+        # NetBSD 6 uses 64-bit time_t on all architectures
+        my $netbsd6;
+        if ( $Config{osname} eq 'netbsd' ) {
+            $Config{osvers} =~ /^([0-9]+)/;
+            if ( $1 and $1 >= 6 ) {
+                $netbsd6 = 1;
+            }
+        }
+        if ( $netbsd6 and $Config{longsize} == 4 ) {
             if ( defined $Config{use64bitint} ) {
                 $timeout = pack( 'QL', $self->{timeout}, 0 );
             }
@@ -376,7 +389,7 @@ sub _recv_data_nb {
         elsif ( $buf ne '' ) {
 
             # received some data
-            $self->{_parser}->add($buf);
+            $self->{_parser}->parse($buf);
         }
         else {
             delete $self->{_socket};
@@ -473,7 +486,7 @@ sub send_command {
     $self->_recv_data_nb;
 
     my $request = $self->{_parser}->build_request( $command, @_ );
-    $self->{_parser}->add_callback($callback);
+    $self->{_parser}->push_callback($callback);
     {
         local $SIG{PIPE} = 'IGNORE' unless $NOSIGNAL;
         defined $self->{_socket}->send( $request, $NOSIGNAL )
@@ -560,19 +573,26 @@ method may block for indefinite time.
 sub mainloop {
     my $self = shift;
 
+    return unless $self->{_parser};
+
     while ( $self->{_parser}->callbacks ) {
         croak "You can't call mainloop in the child process" unless $self->{_pid} == $$;
         my $ret = $self->{_socket}->recv( my $buffer, 131072 );
         unless ( defined $ret ) {
             next if $! == EINTR;
-
-            # TODO: if not EAGAIN then invoke _on_disconnect
-            confess "Error reading reply from server: $!";
+            if ( $! == EAGAIN ) {
+                confess "Timed out waiting reply from the server";
+            }
+            else {
+                $self->_on_disconnect( 1,
+                    RedisDB::Error::DISCONNECTED->new("Error reading reply from the server: $!") );
+                next;
+            }
         }
         if ( $buffer ne '' ) {
 
             # received some data
-            $self->{_parser}->add($buffer);
+            $self->{_parser}->parse($buffer);
         }
         else {
 
@@ -619,7 +639,7 @@ sub get_reply {
         elsif ( $buffer ne '' ) {
 
             # received some data
-            $self->{_parser}->add($buffer);
+            $self->{_parser}->parse($buffer);
         }
         else {
 
@@ -1103,8 +1123,7 @@ sub subscription_loop {
     my ( $self, %args ) = @_;
     croak "Already in subscription loop" if $self->{_subscription_loop};
     croak "You can't start subscription loop while in pipelining mode."
-      if $self->{_parser}->callbacks
-      or @{ $self->{_replies} };
+      if $self->replies_to_fetch;
     $self->{_subscribed}        = {};
     $self->{_psubscribed}       = {};
     $self->{_subscription_cb}   = $args{default_callback};
